#include "interface-gui-events.h"

void saveImage_ev(GtkWidget *widget, gpointer data){

	GtkFileChooser *dialog;
	
	dialog = (GtkFileChooser*)gtk_file_chooser_dialog_new (
					"Save Image",
				      (GtkWindow*)intf_window,
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);

	gtk_file_chooser_set_do_overwrite_confirmation((GtkFileChooser*)dialog, TRUE);
	gtk_file_chooser_set_current_name(dialog, "New image.png");

	if (gtk_dialog_run((GtkDialog*)dialog) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename(dialog);
		util_save_image(filename);
		g_free(filename);
	}
	
	gtk_widget_destroy((GtkWidget*)dialog);

}

void saveSettings_ev(GtkWidget *widget, gpointer data){

	GtkFileChooser *dialog;
	
	dialog = (GtkFileChooser*)gtk_file_chooser_dialog_new (
					"Save Settings",
				      (GtkWindow*)intf_window,
				      GTK_FILE_CHOOSER_ACTION_SAVE,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,
				      NULL);

	gtk_file_chooser_set_do_overwrite_confirmation((GtkFileChooser*)dialog, TRUE);
	gtk_file_chooser_set_current_name(dialog, "My settings");

	if (gtk_dialog_run((GtkDialog*)dialog) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename(dialog);
		util_save_settings(filename);
		g_free(filename);
	}
	
	gtk_widget_destroy((GtkWidget*)dialog);

}

void loadSettings_ev(GtkWidget *widget, gpointer data){

	GtkFileChooser *dialog;
	
	dialog = (GtkFileChooser*)gtk_file_chooser_dialog_new (
					"Load Settings",
				      (GtkWindow*)intf_window,
				      GTK_FILE_CHOOSER_ACTION_OPEN,
				      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
				      GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT,
				      NULL);

	if (gtk_dialog_run((GtkDialog*)dialog) == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename(dialog);
		util_load_settings(filename);
		g_free(filename);
	}
	
	gtk_widget_destroy((GtkWidget*)dialog);

}

void destroy_ev(GtkWidget *widget, gpointer data){
	main_quit();
}

void moveUp_ev(GtkWidget *widget, gpointer data) {
	parameter.y += parameter.height/(parameter.step*parameter.zoom);
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void moveDown_ev(GtkWidget *widget, gpointer data) {
	parameter.y -= parameter.height/(parameter.step*parameter.zoom);
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void moveLeft_ev(GtkWidget *widget, gpointer data) {
	parameter.x += parameter.width/(parameter.step*parameter.zoom);
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void moveRight_ev(GtkWidget *widget, gpointer data) {
	parameter.x -= parameter.width/(parameter.step*parameter.zoom);
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void moveZoomIn_ev(GtkWidget *widget, gpointer data) {
	parameter.zoom *= 2;
	refresh();
	refresh_values();
}

void moveZoomOut_ev(GtkWidget *widget, gpointer data) {
	parameter.zoom /= 2;
	refresh();
	refresh_values();
}

void about_ev() {
/*
	gchar * artists[] = APP_GUI;
	gchar * documenters[] = APP_DOCS;
*/
	gchar * authors[] = APP_AUTH;
	GdkPixbuf *logopixbuf;

	logopixbuf = gdk_pixbuf_new_from_file("imgs/logo.png",NULL);

	if (!logopixbuf) {
		g_message ("Could not load the image!");
		exit (EXIT_FAILURE);
	}

	gtk_show_about_dialog((GtkWindow*)intf_window,
/*		"artists", artists,
*/		"authors",authors,
		"comments",APP_COMM,
		"copyright",APP_COPY,
/*		"documenters",documenters,
*/		"license",APP_LIC,
		"logo",logopixbuf,
		"name",APP_NAME,
/*		"translator-credits",APP_TRANS,
*/		"version",APP_VER,
		"website",APP_WEB,
/*		"website-label",APP_WEB_LABEL,
*/		"wrap-license",TRUE,
		NULL);
}

void reset_ev(GtkWidget *widget, gpointer data) {
	reset(0);
}

void reset_move_ev(GtkWidget *widget, gpointer data) {
	reset(1);
}

void reset_zoom_ev(GtkWidget *widget, gpointer data) {
	reset(2);
}

void value_changed_evA(GtkSpinButton *spinButtonA, gpointer user_data){
	parameter.alpha = gtk_spin_button_get_value(spinButtonA);
	paint_start();
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void value_changed_evB(GtkSpinButton *spinButtonB, gpointer user_data){
	parameter.beta = gtk_spin_button_get_value(spinButtonB);
	paint_start();
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void value_changed_evIter(GtkSpinButton *spinButtonIter, gpointer user_data){
	parameter.iters = gtk_spin_button_get_value(spinButtonIter);
	paint_start();
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void value_changed_evZoom(GtkSpinButton *spinButtonZoom, gpointer user_data){
	parameter.zoom = gtk_spin_button_get_value(spinButtonZoom);
	paint_start();
	refresh();
	gtk_widget_queue_draw(intf_darea);
}

void change_fractal_ev(GtkWidget *widget, gpointer data) {
	int i;
	
	for (i = 0; i < FRACTALS; i++)
		if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(mniFractalItem[i])) == TRUE) {
			parameter.fractal = i+1;
			reset(0);
			paint_start();
			refresh();
			gtk_widget_queue_draw(intf_darea);
			refresh_values();
			break;
		}
}

void color_ev(GtkWidget *widget, gpointer data) {
	parameter.color = gtk_toggle_button_get_active((GtkToggleButton*)widget);
	paint_start();
	refresh();
	gtk_widget_queue_draw(intf_darea);
	refresh_values();
}

void color_from_menu_ev(GtkWidget *widget, gpointer data) {
	parameter.color = gtk_check_menu_item_get_active((GtkCheckMenuItem*)widget);
	refresh();
	gtk_widget_queue_draw(intf_darea);
	refresh_values();
}
