#include "utilities.h"


int util_save_image_bmp(char * name) {
	/* prevzato z rootu a upraveno pro BMP*/
	FILE *fp;
	int file_length;
	int i, j, k;
	unsigned char * padding;
	int padding_per_line;
	unsigned char bmpHeader[54]={		/* hlavicka formatu typu BMP */
		0x42, 0x4D,						/*  0 signature, must be 4D42 hex */
		0x00, 0x00, 0x00, 0x00,			/*  2 size of BMP file in bytes (unreliable) */
		0x00, 0x00,						/*  6 reserved, must be zero */
		0x00, 0x00,						/*  8 reserved, must be zero */
		0x36, 0x00, 0x00, 0x00,			/* 10 offset to start of image data in bytes */
		0x28, 0x00, 0x00, 0x00,			/* 14 size of BITMAPINFOHEADER structure, must be 40 */
		0x00, 0x00, 0x00, 0x00,			/* 18 image width in pixels */
		0x00, 0x00, 0x00, 0x00,			/* 22 image height in pixels */
		0x01, 0x00,						/* 26 number of planes in the image, must be 1 */
		0x18, 0x00,						/* 28 number of bits per pixel (1, 4, 8, or 24) */
		0x00, 0x00, 0x00, 0x00,			/* 30 compression type (0=none, 1=RLE-8, 2=RLE-4) */
		0x00, 0x00, 0x00, 0x00,			/* 34 size of image data in bytes (including padding) */
		0x12, 0x0B, 0x00, 0x00,			/* 38 horizontal resolution in pixels per meter (unreliable) */
		0x12, 0x0B, 0x00, 0x00,			/* 42 vertical resolution in pixels per meter (unreliable) */
		0x00, 0x00, 0x00, 0x00,			/* 46 number of colors in image, or zero */
		0x00, 0x00, 0x00, 0x00,			/* 50 number of important colors, or zero */
	};

	if ((rgbBuffer == NULL) || (limit == 0)) {
		intf_alert("An error occured during saving image.");
		return FALSE;
	}
	if (parameter.width*parameter.height*3 > limit) {
		intf_alert("An error occured during saving image.");
		return FALSE;
	}

	padding_per_line = 4 - parameter.width%4;
	padding = (unsigned char *)calloc(padding_per_line*sizeof(char), 1);
	file_length = 54 + limit + padding_per_line*parameter.height;
	memcpy(bmpHeader+2,  &file_length,		  4);	/* do hlavicky BMP zapsat velikost souboru */
	memcpy(bmpHeader+18, &(parameter.width),  4);	/* do hlavicky BMP zapsat rozmery obrazku */
	memcpy(bmpHeader+22, &(parameter.height), 4);

	fp = fopen(name, "wb");
	if (fp == NULL)
		intf_alert("An error occured during saving image.");
	else {
		fwrite(bmpHeader, 54, 1, fp);			/* zapsat hlavicku BMP do souboru */
		for (j = parameter.height; j ; j--) {
			unsigned int yoff=(j-1)*3*parameter.width;
			unsigned int xoff=0;
			for (i = 0; i < parameter.width; i++) {
				for (k = 0; k < 3; k++) {
					fputc(rgbBuffer[xoff+yoff+2-k], fp);
				}
				xoff += 3;
			}
			fwrite(padding, padding_per_line, 1, fp);
		}
		fclose(fp);
	}
	return TRUE;
}

int util_save_image_tga(char * name) {
	/* prevzato z rootu a upraveno*/
	FILE *fp;
	unsigned int i, j, k;
	unsigned char tgaHeader[18]={		/* hlavicka formatu typu TGA */
				0x00,					/* typ hlavicky TGA */
				0x00,					/* nepouzivame paletu */
				0x02,					/* typ obrazku je RGB TrueColor */
				0x00, 0x00,				/* delka palety je nulova */
				0x00, 0x00, 0x00,		/* pozice v palete nas nezajima */
				0x00, 0x00, 0x00, 0x00,	/* obrazek je umisteny na pozici [0, 0] */
				0x00, 0x00, 0x00, 0x00,	/* sirka a vyska obrazku (dva byty na polozku) */
				0x18,					/* format je 24 bitu na pixel */
				0x20					/* orientace bitmapy v obrazku */
	};

	if ((rgbBuffer == NULL) || (limit == 0)) {
		intf_alert("An error occured during saving image.");
		return FALSE;
	}
	if (parameter.width*parameter.height*3 > limit) {
		intf_alert("An error occured during saving image.");
		return FALSE;
	}

	memcpy(tgaHeader+12, &(parameter.width), 2);	/* do hlavicky TGA zapsat velikost obrazku */
	memcpy(tgaHeader+14, &(parameter.height), 2);

	fp = fopen(name, "wb");
	if (fp == NULL)
		intf_alert("An error occured during saving image.");
	else {
		fwrite(tgaHeader, 18, 1, fp);
		i = parameter.height*parameter.width*3;
		for (j = 0; j < i; j+=3) {
			for (k = 0; k < 3; k++) {
				fputc(rgbBuffer[j+2-k], fp);
			}
		}


		fclose(fp);
	}
	return TRUE;
}

int util_save_image(char * name) {
	GError **error = NULL;
	GdkPixbuf *buffer = NULL;
	
	int len = strlen(name);

	if (len > 3 && name[len-1] == 'a' && name[len-2] == 'g' && name[len-3] == 't')
		return util_save_image_tga(name);
	if (len > 3 && name[len-1] == 'p' && name[len-2] == 'm' && name[len-3] == 'b')
		return util_save_image_bmp(name);

	/* refresh(); */
	
	buffer = gdk_pixbuf_get_from_drawable(NULL, intf_darea->window, gtk_widget_get_colormap(intf_darea),0,0,0,0,parameter.width, parameter.height);
	
	if (buffer == NULL)
		intf_alert("An error occured during saving image.");
	else
		gdk_pixbuf_save(buffer, name, "png", error, NULL);

	return 0;
}

int util_save_settings(char * jmeno) {
	FILE *fp;
	int par_ver = PARAMETER_VERSION;
	
	fp = fopen(jmeno, "wb");
	if (fp == NULL)
		intf_alert("An error occured during saving settings.");
	else {
		fwrite(&par_ver, sizeof(int), 1, fp);
		fwrite(&parameter, sizeof(parameter), 1, fp);
		fclose(fp);
	}

	return 0;
}

int util_load_settings(char * jmeno) {
	FILE *fp;
	int par_ver;

	fp = fopen(jmeno, "rb");
	if (fp == NULL)
		intf_alert("An error occured during load settings.");
	else {
		fread(&par_ver, sizeof(int), 1, fp);
		if (par_ver == PARAMETER_VERSION) {
			fread(&parameter, sizeof(parameter), 1, fp);
			refresh_values();
		} else
			intf_alert("Selected settings was saved in another version of the program.\nLoading aborted.");
		fclose(fp);
	}

	return 0;
}

void recalc_coords(double *x, double *y) {
	*x += parameter.x;
	*y += parameter.y;
	
	*x *= parameter.zoom;
	*y *= parameter.zoom;

	*x += parameter.width/2;
	*y += parameter.height/2;
}

void recalc_coords_rev(double *x, double *y) {
	*x -= parameter.width/2;
	*y -= parameter.height/2;

	*x /= parameter.zoom;
	*y /= parameter.zoom;

	*x -= parameter.x;
	*y -= parameter.y;
}
